import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.*;
import javax.swing.*;

public class ConfigFrame extends Frame 
                   implements ActionListener, ItemListener {
  svserver svServer;             // server side
  Configuration target;      

  ConfigFrame config;            // this object

  JTabbedPane tabbedPane;

  MainPanel rankpanel;            // main panel without buttons
  List listBox;                   // list
  Label description;              // description label below the list
  CardLayout card;                // card
  JPanel opt;                     // option panel main

  OptPanel_0 optp0;               // in case yes and no input
  OptPanel_1 optp1;               // file selection dialog needed?
  OptPanel_2 optp2;               // to read string value
  OptPanel_3 optp3;               // to read int value
  OptPanel_4 optp4;               // to read choice value

  int rank = 1;
  ConfigItem selectedItem;  

  /**
   * constructor
   */
  public ConfigFrame(svserver server) {
     super("Scivis configuration");
     svServer = server;
     target = server.configHandler;
     config = this;

     // initialize the configuration items
     initialize();

     setLayout(new BorderLayout());

     // Create a tab pane
     tabbedPane = new JTabbedPane();
     tabbedPane.addTab("System", rankpanel = new MainPanel());
     tabbedPane.addTab("Rank 1", rankpanel);
     tabbedPane.addTab("Rank 2", rankpanel);
     tabbedPane.addTab("Rank 3", rankpanel);
     tabbedPane.setSelectedIndex(1);
     add("Center", tabbedPane);
     
     // Add Tab change listener
     createTabListener();

     // create button panel
     Panel p = new Panel();
     JButton btn;

     p.setLayout(new FlowLayout(FlowLayout.RIGHT));
     p.add(btn = new JButton("Ok"));
     btn.addActionListener(this);
     p.add(btn = new JButton("Cancel"));
     btn.addActionListener(this);
     add("South", p);
     validate();
  }


  /** 
   * do work when button clicked
   */
  public void actionPerformed(ActionEvent e) {

     String command = e.getActionCommand();

     if (command.equals("Ok")) {
        try {
           String setting = target.getConfigFile();
           File oldfile = new File(setting);
           File oldenough = new File(setting + ".old");
           if (oldenough.exists()) {
               oldenough.delete();
               oldenough = new File(setting + ".old");
           }
           oldfile.renameTo(oldenough);

           Properties p = new Properties();

           for (int i=0; i<ge_items.length; i++) {
              if (ge_items[i].value == null)   
                 p.put(ge_items[i].definition, "");
              else           
                 p.put(ge_items[i].definition, ge_items[i].value);
           }

           for (int i=0; i<r1_items.length; i++) {
              if (r1_items[i].value == null)   
                 p.put(r1_items[i].definition, "");
              else           
                 p.put(r1_items[i].definition, r1_items[i].value);
           }

           for (int i=0; i<r2_items.length; i++) {
              if (r2_items[i].value == null)   
                 p.put(r2_items[i].definition, "");
              else           
                 p.put(r2_items[i].definition, r2_items[i].value);
           }

           /////////////////////////////////////////
           // 3d properties shouls also saved
           /////////////////////////////////////////

           p.save(new FileOutputStream(setting), "Generated by Scivis.");

        } catch (Exception E) {
            svServer.message("Can't update configuration file.", true);
            return;
        }
        svServer.configHandler.reconfigure();
        svServer.fmanager.globalCheck(target.isGlobalMode());


     } 

     svServer.mainb.configBtn.setEnabled(true);
     svServer.config_frame = null;
     this.dispose();
  }

  /** 
   * do work when list item chosen
   */
  public void itemStateChanged(ItemEvent e) {
     if (e.getStateChange() == ItemEvent.SELECTED) 
        listItemChosenAction();
  }

 
  /** 
   * do work when list item chosen(actual worker)
   */
  void listItemChosenAction() {
     int idx = listBox.getSelectedIndex();

     switch (rank) {
        case 0:                 // general
           selectedItem = ge_items[idx];
           break;
        case 1:                 // rank 1
           selectedItem = r1_items[idx];
           break;
        case 2:                 // rank 2
           selectedItem = r2_items[idx];
           break;
        case 3:                 // rank 3
           selectedItem = r3_items[idx];
           break;
     }

     if (selectedItem == null) return;

     switch (selectedItem.type) {
        case YES_NO: 
           card.show(opt, "yesno");
           optp0.title.setText(selectedItem.definition);
           optp0.updateSetting();
           break;
        case BROWSER_GET: 
           card.show(opt, "browser_get");
           optp1.title.setText(selectedItem.definition);   
           optp1.updateSetting();
           break;
        case GET_STR:
           card.show(opt, "get_str");
           optp2.title.setText(selectedItem.definition);   
           optp2.updateSetting();
           break;
        case GET_INT:      
           card.show(opt, "get_int");
           optp3.title.setText(selectedItem.definition);   
           optp3.updateSetting();
           break;
        case CHOICE_GET:      
           card.show(opt, "choice_get");
           optp4.title.setText(selectedItem.definition);   
           optp4.resetComboBox();
           break;      
     }
     description.setText(selectedItem.explanation);
  }


  /**
   * do work when changed tab 
   */
  void createTabListener() {
	// add listener to know when we've been shown
        ChangeListener changeListener = new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JTabbedPane tab = (JTabbedPane) e.getSource();
                rank = tab.getSelectedIndex();
                MainPanel current = (MainPanel)tab.getComponentAt(rank);
                current.changeRank();
            }
        };
        tabbedPane.addChangeListener(changeListener);
  }


  ///////////////////////////////////////////////////////////////
  class MainPanel extends JPanel {

    MainPanel() {

       setLayout(new BorderLayout());
       setBorder(sv.ui.SVUIC.systemBorder);

       Panel cpanel = new Panel();
       cpanel.setLayout(new GridLayout(1, 2));

       JPanel p = new JPanel();
       p.setLayout(new BorderLayout());
       p.setBorder(sv.ui.SVUIC.emptyBorder5);
       p.add("Center", listBox = new List());
       listBox.addItemListener(config);

       opt = new JPanel();
       opt.setBorder(new CompoundBorder(sv.ui.SVUIC.lightLoweredBorder, sv.ui.SVUIC.emptyBorder15)); 
       opt.setLayout(card = new CardLayout());
       opt.add(optp0 = new OptPanel_0(), "yesno"); 
       opt.add(optp1 = new OptPanel_1(), "browser_get");
       opt.add(optp2 = new OptPanel_2(), "get_str"); 
       opt.add(optp3 = new OptPanel_3(), "get_int");
       opt.add(optp4 = new OptPanel_4(), "choice_get"); 

       cpanel.add(p);
       cpanel.add(opt);     

       add("Center", cpanel); 
       add("South", description = new Label("Choose configuration item"));
       description.setForeground(Color.blue);
       resetListItems();
    }

    void changeRank() {
       selectedItem = null;

       if (rank == 0)
          description.setText("General configuration.");
       else
          description.setText("Rank " + rank + " configuration.");

       resetListItems();
       rankpanel.validate();
    }

    void resetListItems() {
       if (listBox.getItemCount() > 0)
          listBox.removeAll();

       if (rank == 0) {
          for (int i=0; i<ge_items.length; i++) 
             listBox.addItem(ge_items[i].definition);

       } else if (rank == 1) {
          for (int i=0; i<r1_items.length; i++) 
             listBox.addItem(r1_items[i].definition);
       } else if (rank == 2) {
          for (int i=0; i<r2_items.length; i++) 
             listBox.addItem(r2_items[i].definition);
       } else if (rank == 3) {
          for (int i=0; i<r3_items.length; i++) 
             listBox.addItem(r3_items[i].definition);
       } 

       if (listBox.getItemCount() > 0) {
          listBox.select(0);
          listItemChosenAction();
       }
    }
    
  } // end of class MainPanel


  //////////////////////////////////////////////////////////////////////
  // yes or noinput
  class OptPanel_0 extends JPanel {
    JLabel title;
    JRadioButton yesbtn, nobtn;

    OptPanel_0() {
       setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));     
       add(title = new JLabel("title is placed here"));    
       add(Box.createRigidArea(new Dimension(1, 20)));

       ButtonGroup group = new ButtonGroup();
       JRadioButton b;

       add(yesbtn = new JRadioButton("Yes"));
       yesbtn.addItemListener(yesnoListener);
       group.add(yesbtn);

       add(nobtn = new JRadioButton("No"));
       nobtn.addItemListener(yesnoListener);
       group.add(nobtn);
    }

    void updateSetting() {
      if (selectedItem != null && selectedItem.value != null) {
         if (selectedItem.value.equals("yes"))
            yesbtn.setSelected(true);
         else
            nobtn.setSelected(true);
        
      }
    }

    ItemListener yesnoListener = new ItemListener() {
       public void itemStateChanged(ItemEvent e) {
          JRadioButton rb = (JRadioButton) e.getSource();
          if(rb.isSelected()) {
             if(rb.getActionCommand().equals("Yes")) {
                selectedItem.value = "yes";
                description.setText("Set configuration: yes");
             } else {
                selectedItem.value = "no";
                description.setText("Set configuration: no");
             }
          }
       }
    };
  }

  //////////////////////////////////////////////////////////////////////
  // browser version get
  class OptPanel_1 extends JPanel {
    JLabel title, comment;
    JTextField field;
    JButton btn;

    OptPanel_1() {
       setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));     
       add(title = new JLabel("title is placed here"));    
       add(Box.createRigidArea(new Dimension(1, 20)));

       add(comment = new JLabel("Enter the value: "));
       add(field = new JTextField());
       add(btn = new JButton("Browse ..."));

       field.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
             selectedItem.value = field.getText().trim();
             description.setText("Set configuration: " + field.getText()); 
          }
       });

       btn.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
             FileDialog openFileDialog = new FileDialog(config,
                              "Choose the file", FileDialog.LOAD);
             openFileDialog.show();

             String fname = openFileDialog.getFile();
             if (fname == null) 
                 return;

             fname = openFileDialog.getDirectory() + fname;
             field.setText(fname);
          }
       });
    }

    void updateSetting() {
      if (selectedItem != null && selectedItem.value != null) 
         field.setText(selectedItem.value);
    }
  }

  //////////////////////////////////////////////////////////////////////
  // just get text
  class OptPanel_2 extends JPanel {
    JLabel title, comment;
    JTextField field;

    OptPanel_2() {
       setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));     
       add(title = new JLabel("title is placed here"));    
       add(Box.createRigidArea(new Dimension(1, 20)));

       add(comment = new JLabel("Enter the value: "));
       add(field = new JTextField());

       field.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
             selectedItem.value = field.getText().trim();
             description.setText("Set configuration: " + field.getText());
          }
       });
    }

    void updateSetting() {
      if (selectedItem != null && selectedItem.value != null) 
         field.setText(selectedItem.value);
    }
  }

  //////////////////////////////////////////////////////////////////////
  // just get integer
  class OptPanel_3 extends JPanel {
    JLabel title, comment;
    JTextField field;

    OptPanel_3() {
       setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));     
       add(title = new JLabel("title is placed here"));    
       add(Box.createRigidArea(new Dimension(1, 20)));

       add(comment = new JLabel("Enter the value: "));
       add(field = new JTextField());

       field.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
 	     try {
                int test = Integer.valueOf(field.getText().trim()).intValue();
                selectedItem.value = String.valueOf(test);
                description.setText("Set configuration: " + field.getText());
             } catch (Exception E) {
                description.setForeground(Color.red);
                description.setText("Input should be integer value");
                description.setForeground(Color.blue);
             }
          }
       });
    }

    void updateSetting() {
      if (selectedItem != null && selectedItem.value != null) 
         field.setText(selectedItem.value);
    }
  }

  //////////////////////////////////////////////////////////////////////
  // get from choice
  // it should be very general... but due to time deadline ...
  // furthermode, there is a bug in Swing
  class OptPanel_4 extends JPanel {
    JLabel title, comment;
    JComboBox choice;

    OptPanel_4() {
       setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));     
       add(title = new JLabel("title is placed here"));    
       add(Box.createRigidArea(new Dimension(1, 20)));

       add(comment = new JLabel("Choose the value: "));
       add(choice = new JComboBox());
    }

    void resetComboBox() {

       if (choice.getItemCount() > 0) 
          choice.removeAllItems();

       if (rank == 2) {
          choice.addItem("Wireframe");
          choice.addItem("Colored wireframe");
          choice.addItem("Hidden surface eliminated");
          choice.addItem("Color mapped");
          choice.addItem("Gray scaled");
          choice.addItem("Point plot");
          choice.addItem("Contour");

       } else if (rank == 3) {
          choice.addItem("Wireframe");
          choice.addItem("Filled");
          choice.addItem("Shading");
       }
       choice.showPopup();
    }

  }

  static final int YES_NO       = 0;
  static final int BROWSER_GET  = 1;
  static final int GET_STR      = 2;
  static final int GET_INT      = 3;
  static final int CHOICE_GET   = 4;

  //////////////////////////////////////////////////////////////////////
  class ConfigItem {
     String definition;             // config item definition
     int type;                      // config item type
     String value = null;           // config item value
     String explanation;            // detailed description 

     /**
      * define each configuration item
      */
      ConfigItem(String def, int t, String explain) {
         definition = def;
         type = t;
         explanation = explain;
      }

      void changeValue(String val) { 
         value = val;
      }

      public String toString() {
         return definition;
      }
  }

  // initialize config item
  void initialize() {
    // config 1d properties
    if (target.config1d.connect_point) r1_items[0].changeValue("yes");
    else                               r1_items[0].changeValue("no");

    if (target.config1d.amr)           r1_items[1].changeValue("yes");
    else                               r1_items[1].changeValue("no");

    if (target.config1d.color_by_info) r1_items[2].changeValue("yes");
    else                               r1_items[2].changeValue("no");

    if (target.config1d.color_by_set)  r1_items[3].changeValue("yes");
    else                               r1_items[3].changeValue("no");

    if (target.config1d.tickmark)      r1_items[4].changeValue("yes");
    else                               r1_items[4].changeValue("no");

    if (target.config1d.display_label) r1_items[5].changeValue("yes");
    else                               r1_items[5].changeValue("no");

    if (target.config1d.display_legend) r1_items[6].changeValue("yes");
    else                               r1_items[6].changeValue("no");

    r1_items[7].changeValue(target.config1d.colormap);
    r1_items[8].changeValue(target.config1d.xLabel);
    r1_items[9].changeValue(target.config1d.yLabel);

    if (target.config1d.isXLabelRotate) r1_items[10].changeValue("yes");
    else                               r1_items[10].changeValue("no");

    if (target.config1d.isYLabelRotate) r1_items[11].changeValue("yes");
    else                               r1_items[11].changeValue("no");

    if (target.config1d.global_mode)   r1_items[12].changeValue("yes");
    else                               r1_items[12].changeValue("no");

    if (target.config1d.overlay_mode)  r1_items[13].changeValue("yes");
    else                               r1_items[13].changeValue("no");

    if (target.config1d.discard_mode)  r1_items[14].changeValue("yes");
    else                               r1_items[14].changeValue("no");

    if (target.config1d.display_time)  r1_items[15].changeValue("yes");
    else                               r1_items[15].changeValue("no");

    r1_items[16].changeValue(String.valueOf(target.config1d.bgColor.getRed()));
    r1_items[17].changeValue(String.valueOf(target.config1d.bgColor.getGreen()));
    r1_items[18].changeValue(String.valueOf(target.config1d.bgColor.getBlue()));

    // config 2d properties
    r2_items[0].changeValue(String.valueOf(target.config2d.plotmode_2d));

    if (target.config2d.delay_regenerating_2d) r2_items[1].changeValue("yes");
    else                                 r2_items[1].changeValue("no");

    if (target.config2d.bounding_box_2d) r2_items[2].changeValue("yes");
    else                                 r2_items[2].changeValue("no");

    if (target.config2d.xymesh_2d)       r2_items[3].changeValue("yes");
    else                                 r2_items[3].changeValue("no");

    if (target.config2d.xyticks_2d)      r2_items[4].changeValue("yes");
    else                                 r2_items[4].changeValue("no");

    if (target.config2d.zticks_2d)       r2_items[5].changeValue("yes");
    else                                 r2_items[5].changeValue("no");

    if (target.config2d.display_grid_2d) r2_items[6].changeValue("yes");
    else                                 r2_items[6].changeValue("no");
   
    if (target.config2d.vector_plot_2d)  r2_items[7].changeValue("yes");
    else                                 r2_items[7].changeValue("no");

    r2_items[8].changeValue(target.config2d.xLabel);
    r2_items[9].changeValue(target.config2d.yLabel);
    r2_items[10].changeValue(target.config2d.zLabel);
    r2_items[11].changeValue(target.config2d.colormap_2d);

    if (target.config2d.global_mode)   r2_items[12].changeValue("yes");
    else                               r2_items[12].changeValue("no");

    if (target.config2d.overlay_mode)  r2_items[13].changeValue("yes");
    else                               r2_items[13].changeValue("no");

    if (target.config2d.discard_mode)  r2_items[14].changeValue("yes");
    else                               r2_items[14].changeValue("no");

    if (target.config2d.display_time)  r2_items[15].changeValue("yes");
    else                               r2_items[15].changeValue("no");

    r2_items[16].changeValue(String.valueOf(target.config2d.bgColor.getRed()));
    r2_items[17].changeValue(String.valueOf(target.config2d.bgColor.getGreen()));
    r2_items[18].changeValue(String.valueOf(target.config2d.bgColor.getBlue()));

    // config 3d properties
    // NOTHING TO YET

    // config system properties
    ge_items[0].changeValue(target.getBrowserName());
    ge_items[1].changeValue(target.getFilterHome());
    if (target.isGlobalMode()) ge_items[2].changeValue("yes");
    else                       ge_items[2].changeValue("no");
    
  }


  ConfigItem[] ge_items = {
     new ConfigItem("brwoser_name", BROWSER_GET, "Web browser name and path"),
     new ConfigItem("filter_home_directory", BROWSER_GET, "Filter root directory"),
     new ConfigItem("command_broadcast", YES_NO, "Will the operation apply to all other window?"),
  };

  ConfigItem[] r1_items = {
     new ConfigItem("connect_points_1d", YES_NO, "Will it connect each data point"),
     new ConfigItem("color_by_amr_1d", YES_NO, "Will it use different color by AMR"),
     new ConfigItem("color_by_info_1d", YES_NO, "Will it use different color by color information by the user"),
     new ConfigItem("color_by_set_1d", YES_NO, "Will it use different color by data set"),
     new ConfigItem("tickmark_1d", YES_NO, "Will it display tickmark?"),
     new ConfigItem("display_label_1d", YES_NO, "Will it display label?"),
     new ConfigItem("display_legend_1d", YES_NO, "Will it display legend?"),
     new ConfigItem("colormap_1d", GET_STR, "Color map file name"),
     new ConfigItem("x_label_1d", GET_STR, "What is the x-axis label?"),
     new ConfigItem("y_label_1d", GET_STR, "What is the y-axis label?"),
     new ConfigItem("x_label_rotate_1d", YES_NO, "Will it ratate x label 90 degree?"),
     new ConfigItem("y_label_rotate_1d", YES_NO, "Will it ratate y label 90 degree?"),
     new ConfigItem("global_mode_1d", YES_NO, "Will it use global min and max?"),
     new ConfigItem("overlay_mode_1d", YES_NO, "Will the filter result be overlaid on original data?"),
     new ConfigItem("discard_mode_1d", YES_NO, "Will the filter operation discard original data set?"),
     new ConfigItem("display_time_1d", YES_NO, "Will it display time value?"),
     new ConfigItem("bgColor_1d_red", GET_INT, "Background color: red component value"),
     new ConfigItem("bgColor_1d_green", GET_INT, "Background color: green component value"),
     new ConfigItem("bgColor_1d_blue", GET_INT, "Background color: blue component value"),  
  };

  ConfigItem[] r2_items = {
     new ConfigItem("plot_mode_2d", CHOICE_GET, "Default plot mode in the 2D window."),
     new ConfigItem("delay_regenerating_2d", YES_NO, "Do you want to see whole process of repainting?"),
     new ConfigItem("bounding_box_2d", YES_NO, "Will it draw bounding box"),
     new ConfigItem("xymesh_2d", YES_NO, "Will it draw xy mesh?"),
     new ConfigItem("xyticks_2d", YES_NO, "Will it draw xy ticks?"),
     new ConfigItem("zticks_2d", YES_NO, "Will it draw z ticks?"),
     new ConfigItem("display_grid_2d", YES_NO, "Will it draw grid?"),
     new ConfigItem("vector_plot_2d", YES_NO, "Not used yet..."),
     new ConfigItem("x_label_2d", GET_STR, "What is the x-axis label?"),
     new ConfigItem("y_label_2d", GET_STR, "What is the y-axis label?"),
     new ConfigItem("z_label_2d", GET_STR, "What is the z-axis label?"),
     new ConfigItem("colormap_2d", GET_STR, "Color map file name"),
     new ConfigItem("global_mode_2d", YES_NO, "Will it use global min and max?"),
     new ConfigItem("overlay_mode_2d", YES_NO, "Will the filter result be overlaid on original data?"),
     new ConfigItem("discard_mode_2d", YES_NO, "Will the filter operation discard original data set?"),
     new ConfigItem("display_time_2d", YES_NO, "Will it display time value?"),
     new ConfigItem("bgColor_2d_red", GET_INT, "Background color: red component value"),
     new ConfigItem("bgColor_2d_green", GET_INT, "Background color: green component value"),
     new ConfigItem("bgColor_2d_blue", GET_INT, "Background color: blue component value"),  
  };

  ConfigItem[] r3_items = {
     new ConfigItem("plot_mode_3d", CHOICE_GET, "Default plot mode in the 3D window."),
     new ConfigItem("delay_regenerating_3d", YES_NO, "Do you want to see whole process of repainting?"),
     new ConfigItem("bounding_box_3d", YES_NO, "Will it draw bounding box"),
     new ConfigItem("xymesh_3d", YES_NO, "Will it draw xy mesh?"),
     new ConfigItem("xyticks_3d", YES_NO, "Will it draw xy ticks?"),
     new ConfigItem("zticks_3d", YES_NO, "Will it draw z ticks?"),
     new ConfigItem("display_grid_3d", YES_NO, "Will it draw grid?"),
     new ConfigItem("x_label_3d", GET_STR, "What is the x-axis label?"),
     new ConfigItem("y_label_3d", GET_STR, "What is the y-axis label?"),
     new ConfigItem("z_label_3d", GET_STR, "What is the z-axis label?"),
     new ConfigItem("colormap_3d", GET_STR, "Color map file name"),
     new ConfigItem("global_mode_3d", YES_NO, "Will it use global min and max?"),
     new ConfigItem("overlay_mode_3d", YES_NO, "Will the filter result be overlaid on original data?"),
     new ConfigItem("discard_mode_3d", YES_NO, "Will the filter operation discard original data set?"),
     new ConfigItem("display_time_3d", YES_NO, "Will it display time value?"),
     new ConfigItem("bgColor_3d_red", GET_INT, "Background color: red component value"),
     new ConfigItem("bgColor_3d_green", GET_INT, "Background color: green component value"),
     new ConfigItem("bgColor_3d_blue", GET_INT, "Background color: blue component value"),  
  };

} // end of class

